/*! \file BioR.c
 *  \brief some functions...
 *  \version 1
 *  \date Created       : 10/01/05 
 *  \date Last Modified : Time-stamp: <2006-01-30 15:12:33 lucas>
 *  \author A. Lucas
 *  \note Licence: CeCIll
 */


#include "ViennaRNA/fold.h"
#include "ViennaRNA/fold_vars.h"
#include "ViennaRNA/PS_dot.h"
#include <ViennaRNA/part_func.h>
//#include "ViennaRNA/utils.h"
#include <string.h>
#include <R.h>
#include <Rdefines.h>

SEXP foldR (SEXP sequenceR, SEXP temperatureR)
{
  SEXP structureR;
  SEXP energyR;
  SEXP liste ;
  char * structureC ;
  const char * sequenceC ;
  int longueure ;
  double temperatureC;
  float energyC;

  /* Get inputs from R */
  sequenceC = CHAR(STRING_ELT(sequenceR,0));
  temperatureC = REAL(temperatureR)[0];

  /* temperature: from fold_vars.h */
  temperature =  temperatureC;
  update_fold_params();

  longueure = strlen (sequenceC) +1;

  /* Alloc */
  structureC = (char*) malloc(2*longueure * sizeof(char));
  initialize_fold(longueure);

  /* Compute */
  energyC = fold(sequenceC,structureC);

  free_arrays();

  /* Return to R */
  PROTECT(structureR = allocVector(STRSXP, 1));
  SET_STRING_ELT(structureR, 0, mkChar(structureC));
  UNPROTECT(1);

  free(structureC);

  PROTECT(energyR = allocVector(REALSXP, 1));
  REAL(energyR)[0] = (double) energyC;
  UNPROTECT(1);

  PROTECT(liste = NEW_LIST(2));
     SET_VECTOR_ELT(liste, 0,structureR);
     SET_VECTOR_ELT(liste, 1,energyR);
  UNPROTECT(1);


  return liste;

}


void R_dot_plot (char ** sequence, double * t, char ** filename)
{
  int longueure;
  char * structureC ;

  longueure = strlen (*sequence) +1;

  structureC = (char*) malloc(2*longueure * sizeof(char));
  initialize_fold(longueure);
  init_pf_fold(longueure);

  /* temperature: from fold_vars.h */
  temperature = *t;
  update_fold_params();
  update_pf_params(longueure);


  /* compute things... */
  fold(*sequence,structureC);
  pf_fold(*sequence,structureC);


  /* This should be avoid... but if not some
     problems happends... */
  PS_rna_plot(*sequence,structureC,*filename);

  /* dot plot */
  PS_dot_plot(*sequence,*filename);

  /* free all */
  free_arrays();
  free_pf_arrays();

  free(structureC);
}      


void R_rna_plot (char ** sequence, char ** structure, char ** filename)
{
  PS_rna_plot(*sequence,*structure,*filename);
}      
